Sample JSON making from PPS
=============================================================
RR_2RPGood_anyarms_ls.py
--------------------------------------


from rr_tools_18 import get_data, obtain_json_from_rr

run_class_name = "Collisions18"
dataset_name = "/Express/Collisions2018/DQM"
ls_query = "(l.CTPPS45_READY = '1' or l.CTPPS56_READY = '1')"
output_filename = "test.json"


query = "((r.RDA_CMP_RP45_210 = 'GOOD' and r.RDA_CMP_RP45_220 = 'GOOD') or \
        (r.RDA_CMP_RP45_CYL = 'GOOD' and r.RDA_CMP_RP45_210 = 'GOOD') or \
        (r.RDA_CMP_RP45_CYL = 'GOOD' and r.RDA_CMP_RP45_220 = 'GOOD')) or \
        ((r.RDA_CMP_RP56_210 = 'GOOD' and r.RDA_CMP_RP56_220 = 'GOOD') or \
        (r.RDA_CMP_RP56_220 = 'GOOD' and r.RDA_CMP_RP56_CYL = 'GOOD') or \
        (r.RDA_CMP_RP56_210 = 'GOOD' and r.RDA_CMP_RP56_CYL = 'GOOD'))"


obtain_json_from_rr(run_class_name, dataset_name, query, ls_query, output_filename)
=============================================================
rr_tools_18.py
-------------------------------------
import sys
from rhapi import DEFAULT_URL, RhApi

def get_data(q):
   api = RhApi(DEFAULT_URL, debug = False)

   p = {"class": "Collisions18"}
   qid = api.qid(q)

   return api.csv(q, p)

def obtain_json_from_rr(run_class_name, dataset_name, query, ls_query, output_filename):
   run_numbers_query = "select r.RUN_NUMBER from runreg_ctpps.datasets r where \
   r.run_class_name = '{0}' and r.RDA_NAME = '{1}' {2}".format(
   run_class_name, dataset_name, "and (" + query + ")" if query else "")

   run_numbers = [int(x) for x in [y for y in get_data(run_numbers_query).split("\n") if y and y.isdigit()]]

   with open(output_filename, 'w') as f:
       f.write("{")
       sign = ","
       for i, run_number in enumerate(run_numbers):
           lumisections_numbers = "select l.RDR_SECTION_FROM, l.RDR_SECTION_TO from runreg_ctpps.dataset_lumis l \
           where l.RDR_RUN_NUMBER = {0} {1}".format(run_number, "and (" + ls_query + ")" if ls_query else "")
           if i == len(run_numbers)-1:
               sign = "} "
           f.write("\"{0}\": [[{1}]]{2}\n".format(run_number, "], [".join([x for x in get_data(lumisections_numbers).split("\n") if x][1:]), sign))


import os
import pytest
import json


from runregistry.runregistry import (
    get_run,
    get_runs,
    get_dataset_names_of_run,
    get_dataset,
    get_datasets,
    get_lumisections,
    get_oms_lumisections,
    get_lumisection_ranges,
    get_oms_lumisection_ranges,
    get_joint_lumisection_ranges,
    generate_json,
)


common_run_number = 327743
common_dataset_name = "/PromptReco/HICosmics18A/DQM"


def test_with_local_certificate():
    # if os.getenv("ENVIRONMENT") == "development":
    # For this test to pass you must include cert and key in certs/ folder:
    cert = "certs/usercert.pem"
    key = "certs/userkey.pem"
    run = get_run(run_number=common_run_number, cert=(cert, key))
    assert run["run_number"] == common_run_number
    lumisections = get_oms_lumisections(run_number=common_run_number, cert=(cert, key))
    assert len(lumisections) > 0
    # else:
    #     pass


def test_get_run():
    run_number = 328762
    run = get_run(run_number=328762)
    # print(run)
    assert run["run_number"] == run_number
    # Non existent run:
    run_number = 8888888888
    run = get_run(run_number=run_number)
    assert run == None


def test_get_runs():
    # Gets runs between run number 309000 and 310000
    filter_run = {"run_number": {"and": [{">": 309000}, {"<": 310000}]}}
    runs = get_runs(filter=filter_run)
    assert len(runs) > 0
    # Gets runs that contain lumisections that classified DT as GOOD AND lumsiections that classified hcal as STANDBY
    filter_run = {
        "run_number": {"and": [{">": 309000}, {"<": 310000}]},
        "dt-dt": "GOOD"
        # 'hcal': 'STANDBY'
    }

    runs = get_runs(filter=filter_run)
    assert len(runs) > 0
    runs = []

    filter_run = {
        "run_number": {"and": [{">": 309000}, {"<": 310000}]},
        "tracker-strip": "GOOD",
    }
    runs = get_runs(filter=filter_run)
    print(json.dumps(runs))
    assert len(runs) > 0


def test_get_runs_with_ignore_filter():
    filter_run = {
        "run_number": {"and": [{">": 309000}, {"<": 310000}]},
        "oms_attributes.hlt_key": {"like": "%commissioning2018%"},
        "triplet_summary.dt-dt.GOOD": {">": 0},
    }
    runs = get_runs(filter=filter_run, ignore_filter_transformation=True)
    assert len(runs) > 0


def test_get_runs_not_compressed():
    runs = get_runs(
        filter={"run_number": {"and": [{">": 309000}, {"<": 310000}]}, "dt-dt": "GOOD"},
        compress_attributes=False,
    )
    assert len(runs) > 0


def get_runs_with_combined_filter():
    runs = get_runs(
        filter={
            "run_number": {
                "and": [{">": 309000}, {"<": 310000}]
                # },
                # 'hlt_key': {
                #     'like': '%commissioning2018%'
                # },
                # 'significant': {
                #     '=': True
            }
        }
    )
    assert len(runs) > 0


def test_get_dataset_names_of_run():
    dataset_names = get_dataset_names_of_run(run_number=321777)
    assert len(dataset_names) > 0


def test_get_dataset():
    dataset = get_dataset(
        run_number=common_run_number, dataset_name=common_dataset_name
    )
    assert dataset["run_number"] == common_run_number
    assert dataset["name"] == common_dataset_name


def test_get_datasets():
    datasets = get_datasets(
        filter={"run_number": {"and": [{">": 309000}, {"<": 310000}]}}
    )
    assert len(datasets) > 0


def test_get_lumisections():
    lumisections = get_lumisections(common_run_number, common_dataset_name)
    assert len(lumisections) > 0


def test_get_oms_lumisections():
    lumisections = get_oms_lumisections(common_run_number)
    assert len(lumisections) > 0
    dataset_lumisections = get_oms_lumisections(common_run_number, common_dataset_name)
    assert len(dataset_lumisections) > 0


def test_get_lumisection_ranges():
    lumisections = get_lumisection_ranges(common_run_number, common_dataset_name)
    assert len(lumisections) > 0


def test_get_oms_lumisection_ranges():
    lumisections = get_lumisection_ranges(common_run_number, common_dataset_name)
    assert len(lumisections) > 0


def test_get_joint_lumisection_ranges():
    lumisections = get_joint_lumisection_ranges(common_run_number, common_dataset_name)
    assert len(lumisections) > 0


def test_get_collisions18():
    runs = get_runs(filter={"class": "Collisions18"})
    assert len(runs) > 0


def test_get_datasets_with_filter():
    datasets = get_datasets(
        filter={
            "run_number": {"and": [{">": 309000}, {"<": 310000}]},
            "tracker-strip": "GOOD",
        }
    )
    assert len(datasets) > 0


def test_generate_json():
    json_logic = """
        {
        "and": [
            {
                "or": [
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018A/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018B/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018C/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018D/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018E/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018F/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018G/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018H/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018I/DQM"]}
                ]
            },
            { ">=": [{ "var": "run.oms.energy" }, 6000] },
            { "<=": [{ "var": "run.oms.energy" }, 7000] },
            { ">=": [{ "var": "run.oms.b_field" }, 3.7] },
            { "in": [ "25ns", { "var": "run.oms.injection_scheme" }] },
            { "==": [{ "in": [ "WMass", { "var": "run.oms.hlt_key" }] }, false] },

            { "==": [{ "var": "lumisection.rr.dt-dt" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.csc-csc" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.l1t-l1tmu" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.l1t-l1tcalo" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.hlt-hlt" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.tracker-pixel" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.tracker-strip" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.tracker-track" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.ecal-ecal" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.ecal-es" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.hcal-hcal" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.muon-muon" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.jetmet-jetmet" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.lumi-lumi" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.dc-lowlumi" }, "BAD"] },

            { "==": [{ "var": "lumisection.oms.cms_active" }, true] },
            { "==": [{ "var": "lumisection.oms.bpix_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.fpix_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.tibtid_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.tecm_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.tecp_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.tob_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.ebm_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.ebp_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.eem_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.eep_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.esm_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.esp_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.hbhea_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.hbheb_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.hbhec_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.hf_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.ho_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.dtm_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.dtp_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.dt0_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.cscm_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.cscp_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.rpc_ready" }, true] },
            { "==": [{ "var": "lumisection.oms.beam1_present" }, true] },
            { "==": [{ "var": "lumisection.oms.beam2_present" }, true] },
            { "==": [{ "var": "lumisection.oms.beam1_stable" }, true] },
            { "==": [{ "var": "lumisection.oms.beam2_stable" }, true] }
        ]
    }
    """
    final_json = generate_json(json_logic)
    assert final_json is not None
    json_logic2 = {
        "and": [
            {
                "or": [
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018A/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018B/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018C/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018D/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018E/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018F/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018G/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018H/DQM"]},
                    {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018I/DQM"]}
                ]
            },
            { ">=": [{ "var": "run.oms.energy" }, 6000] },
            { "<=": [{ "var": "run.oms.energy" }, 7000] },
            { ">=": [{ "var": "run.oms.b_field" }, 3.7] },
            { "in": [ "25ns", { "var": "run.oms.injection_scheme" }] },
            { "==": [{ "in": [ "WMass", { "var": "run.oms.hlt_key" }] }, False] },

            { "==": [{ "var": "lumisection.rr.dt-dt" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.csc-csc" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.l1t-l1tmu" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.l1t-l1tcalo" }, "GOOD"] },
            { "==": [{ "var": "lumisection.rr.hlt-hlt" }, "GOOD"] },

            { "==": [{ "var": "lumisection.oms.bpix_ready" }, True] }
        ]
    }
    final_json2 = generate_json(json_logic2)


    assert final_json2 is not None

test_generate_json()


import os
import json
import requests
from cernrequests import get_sso_cookies, certs
from runregistry.utils import transform_to_rr_run_filter, transform_to_rr_dataset_filter
import urllib3

# Silence unverified HTTPS warning:
# urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
PAGE_SIZE = 50

cert = "certs/usercert.pem"
key = "certs/userkey.pem"
api_url = "https://cmsrunregistry.web.cern.ch/api"
if os.getenv("ENVIRONMENT") == "development":
    api_url = "http://localhost:9500"
    cert = "certs/usercert.pem"
    key = "certs/userkey.pem"


def _get_cookies(url, **kwargs):
    """
    Gets the cookies required to query RR API
    :return: the cookies required to query Run Registry API. In particular 'connect.sid' is the one we are interested in
    """
    if os.getenv("ENVIRONMENT") == "development":
        return None
    cert = kwargs.pop("cert", None)
    # If no certificate provided, cernrequests will look in default paths:
    cert = cert if cert else certs.default_user_certificate_paths()
    if cert == ("", ""):
        raise Exception(
            'No certificate passed, pass one in a tuple as cert=(cert,key), to authenticate your request. Or place them in your /private folder on AFS under the names of "usercert.pem" and "userkey.pem", please read authentication on README.md for more details'
        )
    ca_bundle = certs.where()
    cookies = get_sso_cookies(url, cert)
    return cookies


def _get_page(
    url, page=0, data_type="runs", ignore_filter_transformation=False, **kwargs
):
    """
    :param ignore_filter_transformation: If user knows how the filter works (by observing http requests on RR website), and wants to ignore the suggested transformation to query API, user can do it by setting ignore_filter_transformation to True
    :param filter: The filter to be transformed into RR syntax, and then sent for querying
    :return: A page in Run registry
    """
    headers = {"Content-type": "application/json"}
    cookies = _get_cookies(url, **kwargs)
    query_filter = kwargs.pop("filter", {})
    if data_type == "runs" and not ignore_filter_transformation:
        query_filter = transform_to_rr_run_filter(run_filter=query_filter)
    elif data_type == "datasets" and not ignore_filter_transformation:
        query_filter = transform_to_rr_dataset_filter(dataset_filter=query_filter)
    if os.getenv("ENVIRONMENT") == "development":
        print(url)
        print(query_filter)
    payload = json.dumps(
        {
            "page": page,
            "filter": query_filter,
            "page_size": kwargs.pop("page_size", PAGE_SIZE),
            "sortings": kwargs.pop("sortings", []),
        }
    )
    return requests.post(url, cookies=cookies, headers=headers, data=payload).json()


def get_dataset_names_of_run(run_number, **kwargs):
    """
    Gets the existing dataset names of a run_number 
    :return: Array of dataset names of the specified run_number
    """
    url = "{}/get_all_dataset_names_of_run/{}".format(api_url, run_number)
    cookies = _get_cookies(url, **kwargs)
    return requests.get(url, cookies=cookies, verify=False).json()


def get_run(run_number, **kwargs):
    """
    Gets all the info about a particular run
    :param run_number: run_number of specified run
    """
    run = get_runs(filter={"run_number": run_number}, **kwargs)
    if len(run) != 1:
        return None
    return run[0]


def get_runs(limit=40000, compress_attributes=True, **kwargs):
    """
    Gets all runs that match the filter given in 
    :param compress_attributes: Gets the attributes inside rr_attributes:* and the ones in the DatasetTripletCache (The lumisections insdie the run/dataset) and spreads them over the run object
    :param filter: the filter applied to the runs needed
    """
    url = "{}/runs_filtered_ordered".format(api_url)
    initial_response = _get_page(url=url, data_type="runs", page=0, **kwargs)
    if "err" in initial_response:
        raise ValueError(initial_response["err"])

    resource_count = initial_response["count"]
    page_count = initial_response["pages"]
    runs = initial_response["runs"]
    if resource_count > limit:
        print(
            "ALERT: The specific run registry api request returns more runs than the limit({}), consider passing a greater limit to get_runs(limit=number) to get the whole result.".format(
                limit
            )
        )
    if resource_count > 10000:
        print(
            "WARNING: fetching more than 10,000 runs from run registry. you probably want to pass a filter into get_runs, or else this will take a while."
        )
    if resource_count > 20000 and "filter" not in kwargs:
        print(
            "ERROR: For run registry queries that retrieve more than 20,000 runs, you must pass a filter into get_runs, an empty filter get_runs(filter={}) works"
        )
        return None
    for page_number in range(1, page_count):
        additional_runs = _get_page(
            page=page_number, url=url, data_type="runs", **kwargs
        )
        runs.extend(additional_runs.get("runs"))
        if len(runs) >= limit:
            runs = runs[:limit]
            break

    if compress_attributes:
        compressed_runs = []
        for run in runs:
            compressed_run = {
                "oms_attributes": run["oms_attributes"],
                **run["rr_attributes"],
                "lumisections": run["DatasetTripletCache"]["triplet_summary"],
                **run,
            }
            del compressed_run["rr_attributes"]
            del compressed_run["DatasetTripletCache"]
            compressed_runs.append(compressed_run)
        return compressed_runs

    return runs


def get_dataset(run_number, dataset_name="online", **kwargs):
    """
    Gets information about the dataset specified by run_number and dataset_name
    :param run_number:  The run number of the dataset
    :param dataset_name: The name of the dataset. 'online' is the dataset of the online run. These are Run Registry specific dataset names e.g. online, /PromptReco/Collisions2018D/DQM, /Express/Collisions2018/DQM 
    """
    dataset = get_datasets(
        filter={"run_number": run_number, "dataset_name": dataset_name}, **kwargs
    )
    if len(dataset) != 1:
        return None
    return dataset[0]


def get_datasets(limit=40000, compress_attributes=True, **kwargs):
    """
    Gets all datasets that match the filter given
    :param compress_attributes: Gets the attributes inside rr_attributes:* and the ones in the DatasetTripletCache (The lumisections insdie the run/dataset) and spreads them over the run object
    """
    url = "{}/datasets_filtered_ordered".format(api_url)
    initial_response = _get_page(url=url, data_type="datasets", page=0, **kwargs)
    if "err" in initial_response:
        raise ValueError(initial_response["err"])

    resource_count = initial_response["count"]
    page_count = initial_response["pages"]
    datasets = initial_response["datasets"]
    if resource_count > limit:
        print(
            "ALERT: The specific api request returns more datasets than the limit({}), consider passing a greater limit to get_datasets(limit=number) to get the whole result.".format(
                limit
            )
        )
    if resource_count > 10000:
        print(
            "WARNING: fetching more than 10,000 datasets. you probably want to pass a filter into get_datasets, or else this will take a while."
        )
    if resource_count > 20000 and "filter" not in kwargs:
        print(
            "ERROR: For queries that retrieve more than 20,000 datasets, you must pass a filter into get_datasets, an empty filter get_datasets(filter={}) works"
        )
        return None
    for page_number in range(1, page_count):
        additional_datasets = _get_page(
            page=page_number, url=url, data_type="datasets", **kwargs
        )
        datasets.extend(additional_datasets.get("datasets"))
        if len(datasets) >= limit:
            datasets = datasets[:limit]
            break

    if compress_attributes:
        compressed_datasets = []
        for dataset in datasets:
            compressed_dataset = {
                **dataset["Run"]["rr_attributes"],
                **dataset,
                "lumisections": dataset["DatasetTripletCache"]["triplet_summary"],
            }
            del compressed_dataset["DatasetTripletCache"]
            del compressed_dataset["Run"]
            compressed_datasets.append(compressed_dataset)
        return compressed_datasets
    return datasets


def _get_lumisection_helper(url, run_number, dataset_name="online", **kwargs):
    """
    Puts the headers, and cookies for all other lumisection methods
    """
    headers = {"Content-type": "application/json"}
    cookies = _get_cookies(url, **kwargs)
    payload = json.dumps({"run_number": run_number, "dataset_name": dataset_name})
    return requests.post(url, cookies=cookies, headers=headers, data=payload).json()


def get_lumisections(run_number, dataset_name="online", **kwargs):
    """
    Gets the Run Registry lumisections of the specified dataset
    """
    url = "{}/lumisections/rr_lumisections".format(api_url)
    return _get_lumisection_helper(url, run_number, dataset_name, **kwargs)


def get_oms_lumisections(run_number, dataset_name="online", **kwargs):
    """
    Gets the OMS lumisections saved in RR database
    """
    url = "{}/lumisections/oms_lumisections".format(api_url)
    return _get_lumisection_helper(url, run_number, dataset_name, **kwargs)


def get_lumisection_ranges(run_number, dataset_name="online", **kwargs):
    """
    Gets the lumisection ranges of the specified dataset
    """
    url = "{}/lumisections/rr_lumisection_ranges".format(api_url)
    return _get_lumisection_helper(url, run_number, dataset_name, **kwargs)


def get_oms_lumisection_ranges(run_number, **kwargs):
    """
    Gets the OMS lumisection ranges of the specified dataset (saved in RR database)
    """
    url = "{}/lumisections/oms_lumisection_ranges".format(api_url)
    return _get_lumisection_helper(url, run_number, dataset_name="online", **kwargs)


def get_joint_lumisection_ranges(run_number, dataset_name="online", **kwargs):
    """
    Gets the lumisection ranges of the specified dataset, breaken into RR breaks and OMS ranges
    """
    url = "{}/lumisections/joint_lumisection_ranges".format(api_url)
    return _get_lumisection_helper(url, run_number, dataset_name, **kwargs)


def generate_json(json_logic, **kwargs):
    """
    It receives a json logic configuration and returns a json with lumisections which pass the filter
    """
    if isinstance(json_logic, str) == False:
        json_logic = json.dumps(json_logic)
    url = "{}/json_creation/generate".format(api_url)
    headers = {"Content-type": "application/json"}
    cookies = _get_cookies(url, **kwargs)
    payload = json.dumps({"json_logic": json_logic})
    response = requests.post(url, cookies=cookies, headers=headers, data=payload).json()
    return response['final_json']
